unit fMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Buttons, Menus, fSyntaxTree, fSimplifiedCode,
  atScript, ShellApi, atParser;

type
  TfmMain = class(TForm)
    Panel2: TPanel;
    MainMenu1: TMainMenu;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    File1: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    Saveas1: TMenuItem;
    Quit1: TMenuItem;
    N1: TMenuItem;
    Show1: TMenuItem;
    SyntaxTree1: TMenuItem;
    Simplifiedcode1: TMenuItem;
    Script1: TMenuItem;
    Help1: TMenuItem;
    Run1: TMenuItem;
    Manual1: TMenuItem;
    About1: TMenuItem;
    N2: TMenuItem;
    FScript: TatPascalScripter;
    RichEdit1: TRichEdit;
    StatusBar1: TStatusBar;
    miScriptInfo: TMenuItem;
    N3: TMenuItem;
    Delphivariablevalue1: TMenuItem;
    procedure Run1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Saveas1Click(Sender: TObject);
    procedure Quit1Click(Sender: TObject);
    procedure SyntaxTree1Click(Sender: TObject);
    procedure Simplifiedcode1Click(Sender: TObject);
    procedure Manual1Click(Sender: TObject);
    procedure RichEdit1SelectionChange(Sender: TObject);
    procedure RichEdit1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure miScriptInfoClick(Sender: TObject);
    procedure Delphivariablevalue1Click(Sender: TObject);
  private
    { Private declarations }
    FScriptName : string;
    procedure SetScriptName(value:string);
    procedure UpdateCurrentPosition;
    procedure GetVarValueProc;
    procedure GetControlCount;
    procedure GetControls;
  public
    { Public declarations }
    property ScriptName:string read FScriptName write SetScriptName;
  end;

var
  fmMain: TfmMain;

implementation

uses fAbout, fScriptInfo{$IFDEF VER140},Variants{$ENDIF};

{$R *.DFM}

procedure TfmMain.SetScriptName(value:string);
begin
   if value<>FScriptName then
   begin
      Caption:='atPascal demo application';
      if value>'' then
         Caption:=Caption+' - '+value;
      FScriptName:=value;
   end;
end;

procedure TfmMain.Run1Click(Sender: TObject);
begin
   with FScript do
   begin
      SourceCode:=RichEdit1.Lines;
      Compile;
      Execute( null );
   end;
end;

procedure TfmMain.About1Click(Sender: TObject);
begin
   fmAbout.ShowModal;
end;

procedure TfmMain.New1Click(Sender: TObject);
begin
   RichEdit1.Lines.Clear;
end;

procedure TfmMain.Open1Click(Sender: TObject);
begin
   with OpenDialog1 do
      if Execute then
      begin
         ScriptName:=FileName;
         RichEdit1.Lines.LoadFromFile( ScriptName );
      end;
end;

procedure TfmMain.Save1Click(Sender: TObject);
begin
   if ScriptName='' then
      Saveas1Click(Sender)
   else
      RichEdit1.Lines.SaveToFile( ScriptName );
end;

procedure TfmMain.Saveas1Click(Sender: TObject);
begin
   with SaveDialog1 do
      if Execute then
      begin
         ScriptName:=FileName;
         RichEdit1.Lines.SaveToFile( ScriptName );
      end;
end;

procedure TfmMain.Quit1Click(Sender: TObject);
begin
   Application.Terminate;
end;

procedure TfmMain.SyntaxTree1Click(Sender: TObject);
begin
   with FScript do
   begin
      SourceCode:=RichEdit1.Lines;
      Compile;
      BuildSyntaxTreeView( fmSyntaxTree.TreeView1 );
   end;
   fmSyntaxTree.ShowModal;
end;

procedure TfmMain.Simplifiedcode1Click(Sender: TObject);
begin
   with FScript do
   begin
      SourceCode:=RichEdit1.Lines;
      Compile;
      BuildSimplifiedCodeListView( fmSimplifiedCode.ListView1 );
   end;
   fmSimplifiedCode.ShowModal;
end;

procedure TfmMain.miScriptInfoClick(Sender: TObject);
begin
   with FScript do
   begin
      SourceCode:=RichEdit1.Lines;
      Compile;
   end;
   fmScriptInfo.ShowInformation( FScript );
end;

procedure TfmMain.Manual1Click(Sender: TObject);
begin
   ShellExecute(0,nil,PChar(ExtractFilePath(Application.ExeName)+'..\Help\atScript.hlp'),nil,nil,SW_SHOWNORMAL);
end;

procedure TfmMain.UpdateCurrentPosition;
var row,col: integer;
begin
   with RichEdit1 do
      GetRowColFromStrings(Lines,SelStart,row,col);
   StatusBar1.Panels[0].Text:=Format('%4d:%4d',[row,col]);
end;

procedure TfmMain.RichEdit1SelectionChange(Sender: TObject);
begin
   UpdateCurrentPosition;
end;

procedure TfmMain.RichEdit1Change(Sender: TObject);
begin
   UpdateCurrentPosition;
end;

var X: variant;

procedure TfmMain.GetVarValueProc;
begin
   with FScript do
   begin
      ReturnOutputArg( FScript.ScriptInfo.Routines[0].Variables.VariableByName( GetInputArgAsString(0) ).Value );
   end;
end;

procedure TfmMain.GetControlCount;
begin
   with FScript, TWinControl( CurrentObject ) do
   begin
      ReturnOutputArg( ControlCount);
   end;
end;

procedure TfmMain.GetControls;
begin
   with FScript, TWinControl( CurrentObject ) do
   begin
      ReturnOutputArg( integer(Controls[GetInputArgAsInteger(0)]) );
   end;
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
   {Add class TWinControl to the known classes list.
    Also adds properties ControlCount and Controls to TWinControl class. Any TWinControl
   descendant object added (with UsesObject or AddObject) will automatically have
   these two properties. ControlCount and Controls properties are not added
   automatically because they're not published but public.

   AddDelphiClass method automatically gets all published properties from TWinControl class.}
   with FScript.AddDelphiClass(TWinControl) do
   begin
      AddMethod('ControlCount',0,tkInteger,nil,GetControlCount);
      AddMethod('Controls',1,tkClass,TControl,GetControls);
   end;

   { Customization of classes known by atPascal. Adding methods Hide, Show, Close and
     ShowModal to TForm object. Now any TFom descendant object added can use Hide, Show,
     Close and ShowModal methods.

     The AddDelphiClass method gets all published properties from TForm, and the
     AddNoArgsNoResultProc method allows using methods in TForm objects}
   with FScript.AddDelphiClass(TForm) do
   begin
      AddNoArgsNoResultProc('Hide',Hide);
      AddNoArgsNoResultProc('Show',Show);
      AddNoArgsNoResultProc('Close',Close);
      AddNoArgsResultIntegerProc('ShowModal',ShowModal);
   end;

   x:=123.45;
   {Allows using PI variable as constant 'PI' in atPascal}
   FScript.SystemLibrary.AddConstant('PI',PI);
   {Allows using X variable as variable 'var' in atPascal}
   FScript.SystemLibrary.AddVariable('X',X);
   {Allows using self object to refer to current form}
   FScript.SystemLibrary.AddObject('self',self);
   {Allows accessing RichEdit1 component}
   FScript.SystemLibrary.AddObject('RichEdit1',RichEdit1);
   {Allows accessing fmAbout object}
   fmAbout:=TfmAbout.Create(Application);
   FScript.SystemLibrary.AddObject('fmAbout',fmAbout);

   {This function exemplifies how to get atPascal script runtime information. In this example,
    GetVarValue returns the value of any atPascal script variable}
   FScript.SystemLibrary.AddMethod('GetVarValue',1,tkInteger,nil,GetVarValueProc);

   {Add self (this form) to default instances. UsesObject method allows accessing properties and
   methods of passed object. When you use UsesObject, it is considered "default". In other words,
   you don't have to use its name to access its properties ( self.caption:='Test', for example ).
   Instead, you can use property directly (Caption:='Test')
   }
   FScript.UsesObject(self);

   { Load default script }
   ScriptName:=ExtractFilePath(Application.ExeName)+'Default.psc';
   RichEdit1.Lines.LoadFromFile( ScriptName );
end;

procedure TfmMain.Delphivariablevalue1Click(Sender: TObject);
begin
   ShowMessage('The value of "X" variable in Delphi is '+FloatToStr(X));
end;

end.
